==> japhar-0.04.tar.gz: japhar-0.04/lib/libnative/java.lang.reflect/field.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   field.c -- native methods for java/lang/reflect/Field
   Created: Chris Toshok <toshok@hungry.com>, 12-Feb-1998.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "ClazzFile.h"
#include "objects.h"
#include "interp.h"

static FieldStruct *
get_field(JNIEnv *env,
	  jobject field)
{
  jint slot;
  jobject clazz;
  ClazzFile *cf, *class_cf = (*env)->FindClass(env, "java/lang/Class");
  obj_thunk *o;
  jclass field_class = (*env)->FindClass(env, "java/lang/reflect/Field");
  jfieldID slot_field = (*env)->GetFieldID(env, field_class, "slot", "I");
  jfieldID clazz_field = (*env)->GetFieldID(env, field_class, "clazz", "Ljava/lang/Class;");

  slot = (*env)->GetIntField(env, field, slot_field);
  clazz = (*env)->GetObjectField(env, field, clazz_field);

  o = find_instance_data_for_class(clazz, class_cf);

  cf = (ClazzFile*)(INSTANCE_DATA(o)->fields[0].value.l);

  return cf->fields[slot];
}

JNIEXPORT jint JNICALL
Java_java_lang_reflect_Field_getModifiers(JNIEnv *env,
					  jobject field)
{
  FieldStruct *f = get_field(env, field);

  return f->access_flags;
}

JNIEXPORT jobject JNICALL
Java_java_lang_reflect_Field_get(JNIEnv *env,
				 jobject field,
				 jobject obj)
{
  (*env)->FatalError(env, "java/lang/reflect/Field.get() not implemented yet.");
  return NULL;
}

JNIEXPORT jboolean JNICALL
Java_java_lang_reflect_Field_getBoolean(JNIEnv *env,
					jobject field,
					jobject obj)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  get_instance_field(obj, f, &item);

  return item.value.z;
}

JNIEXPORT jbyte JNICALL
Java_java_lang_reflect_Field_getByte(JNIEnv *env,
				     jobject field,
				     jobject obj)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  get_instance_field(obj, f, &item);

  return item.value.b;
}

JNIEXPORT jchar JNICALL
Java_java_lang_reflect_Field_getChar(JNIEnv *env,
				     jobject field,
				     jobject obj)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  get_instance_field(obj, f, &item);

  return item.value.c;
}

JNIEXPORT jshort JNICALL
Java_java_lang_reflect_Field_getShort(JNIEnv *env,
				      jobject field,
				      jobject obj)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  get_instance_field(obj, f, &item);

  return item.value.s;
}

JNIEXPORT jint JNICALL
Java_java_lang_reflect_Field_getInt(JNIEnv *env,
				    jobject field,
				    jobject obj)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  get_instance_field(obj, f, &item);

  return item.value.i;
}

JNIEXPORT jlong JNICALL
Java_java_lang_reflect_Field_getLong(JNIEnv *env,
				     jobject field,
				     jobject obj)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  get_instance_field(obj, f, &item);

  return item.value.j;
}

JNIEXPORT jfloat JNICALL
Java_java_lang_reflect_Field_getFloat(JNIEnv *env,
				      jobject field,
				      jobject obj)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  get_instance_field(obj, f, &item);

  return item.value.f;
}

JNIEXPORT jdouble JNICALL
Java_java_lang_reflect_Field_getDouble(JNIEnv *env,
				       jobject field,
				       jobject obj)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  get_instance_field(obj, f, &item);

  return item.value.d;
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_set(JNIEnv *env,
				 jobject field,
				 jobject obj,
				 jobject value)
{
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setBoolean(JNIEnv *env,
					jobject field,
					jobject obj,
					jboolean value)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  item.tag = SIG_JBOOLEAN;
  item.value.z = value;

  set_instance_field(obj, f, item);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setByte(JNIEnv *env,
				     jobject field,
				     jobject obj,
				     jbyte value)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  item.tag = SIG_JBYTE;
  item.value.b = value;

  set_instance_field(obj, f, item);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setChar(JNIEnv *env,
				     jobject field,
				     jobject obj,
				     jchar value)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  item.tag = SIG_JCHAR;
  item.value.c = value;

  set_instance_field(obj, f, item);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setShort(JNIEnv *env,
				      jobject field,
				      jobject obj,
				      jshort value)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  item.tag = SIG_JSHORT;
  item.value.s = value;

  set_instance_field(obj, f, item);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setInt(JNIEnv *env,
				    jobject field,
				    jobject obj,
				    jint value)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  item.tag = SIG_JINT;
  item.value.i = value;

  set_instance_field(obj, f, item);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setLong(JNIEnv *env,
				     jobject field,
				     jobject obj,
				     jlong value)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  item.tag = SIG_JLONG;
  item.value.j = value;

  set_instance_field(obj, f, item);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setFloat(JNIEnv *env,
				      jobject field,
				      jobject obj,
				      jfloat value)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  item.tag = SIG_JFLOAT;
  item.value.f = value;

  set_instance_field(obj, f, item);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setDouble(JNIEnv *env,
				       jobject field,
				       jobject obj,
				       jdouble value)
{
  FieldStruct *f = get_field(env, field);
  JavaStackItem item;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  item.tag = SIG_JDOUBLE;
  item.value.d = value;

  set_instance_field(obj, f, item);
}

==> japhar-0.05.tar.gz: japhar-0.05/lib/libnative/java.lang.reflect/field.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   field.c -- native methods for java/lang/reflect/Field
   Created: Chris Toshok <toshok@hungry.com>, 12-Feb-1998.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "ClazzFile.h"
#include "objects.h"
#include "interp.h"

static FieldStruct *
get_field(JNIEnv *env,
	  jobject field)
{
  jint slot;
  jobject clazz;
  ClazzFile *cf, *class_cf = (*env)->FindClass(env, "java/lang/Class");
  japhar_obj o;
  jclass field_class = (*env)->FindClass(env, "java/lang/reflect/Field");
  jfieldID slot_field = (*env)->GetFieldID(env, field_class, "slot", "I");
  jfieldID clazz_field = (*env)->GetFieldID(env, field_class, "clazz", "Ljava/lang/Class;");
  FieldStruct *cf_field;
  jvalue cf_value;

  slot = (*env)->GetIntField(env, field, slot_field);
  clazz = (*env)->GetObjectField(env, field, clazz_field);

  o = cast_obj(clazz, class_cf);

  cf_field = class_cf->fields[0];

  get_instance_field(o, cf_field, &cf_value);
  cf = (ClazzFile*)cf_value.l;

  return cf->fields[slot];
}

JNIEXPORT jint JNICALL
Java_java_lang_reflect_Field_getModifiers(JNIEnv *env,
					  jobject field)
{
  FieldStruct *f = get_field(env, field);

  return f->access_flags;
}

JNIEXPORT jobject JNICALL
Java_java_lang_reflect_Field_get(JNIEnv *env,
				 jobject field,
				 jobject obj)
{
  (*env)->FatalError(env, "java/lang/reflect/Field.get() not implemented yet.");
  return NULL;
}

JNIEXPORT jboolean JNICALL
Java_java_lang_reflect_Field_getBoolean(JNIEnv *env,
					jobject field,
					jobject obj)
{
  FieldStruct *f = get_field(env, field);
  jvalue value;

  get_instance_field(obj, f, &value);

  return value.z;
}

JNIEXPORT jbyte JNICALL
Java_java_lang_reflect_Field_getByte(JNIEnv *env,
				     jobject field,
				     jobject obj)
{
  FieldStruct *f = get_field(env, field);
  jvalue value;

  get_instance_field(obj, f, &value);

  return value.b;
}

JNIEXPORT jchar JNICALL
Java_java_lang_reflect_Field_getChar(JNIEnv *env,
				     jobject field,
				     jobject obj)
{
  FieldStruct *f = get_field(env, field);
  jvalue value;

  get_instance_field(obj, f, &value);

  return value.c;
}

JNIEXPORT jshort JNICALL
Java_java_lang_reflect_Field_getShort(JNIEnv *env,
				      jobject field,
				      jobject obj)
{
  FieldStruct *f = get_field(env, field);
  jvalue value;

  get_instance_field(obj, f, &value);

  return value.s;
}

JNIEXPORT jint JNICALL
Java_java_lang_reflect_Field_getInt(JNIEnv *env,
				    jobject field,
				    jobject obj)
{
  FieldStruct *f = get_field(env, field);
  jvalue value;

  get_instance_field(obj, f, &value);

  return value.i;
}

JNIEXPORT jlong JNICALL
Java_java_lang_reflect_Field_getLong(JNIEnv *env,
				     jobject field,
				     jobject obj)
{
  FieldStruct *f = get_field(env, field);
  jvalue value;

  get_instance_field(obj, f, &value);

  return value.j;
}

JNIEXPORT jfloat JNICALL
Java_java_lang_reflect_Field_getFloat(JNIEnv *env,
				      jobject field,
				      jobject obj)
{
  FieldStruct *f = get_field(env, field);
  jvalue value;

  get_instance_field(obj, f, &value);

  return value.f;
}

JNIEXPORT jdouble JNICALL
Java_java_lang_reflect_Field_getDouble(JNIEnv *env,
				       jobject field,
				       jobject obj)
{
  FieldStruct *f = get_field(env, field);
  jvalue value;

  get_instance_field(obj, f, &value);

  return value.d;
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_set(JNIEnv *env,
				 jobject field,
				 jobject obj,
				 jobject value)
{
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setBoolean(JNIEnv *env,
					jobject field,
					jobject obj,
					jboolean value)
{
  FieldStruct *f = get_field(env, field);
  jvalue val;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  val.z = value;

  set_instance_field(obj, f, val);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setByte(JNIEnv *env,
				     jobject field,
				     jobject obj,
				     jbyte value)
{
  FieldStruct *f = get_field(env, field);
  jvalue val;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  val.b = value;

  set_instance_field(obj, f, val);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setChar(JNIEnv *env,
				     jobject field,
				     jobject obj,
				     jchar value)
{
  FieldStruct *f = get_field(env, field);
  jvalue val;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  val.c = value;

  set_instance_field(obj, f, val);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setShort(JNIEnv *env,
				      jobject field,
				      jobject obj,
				      jshort value)
{
  FieldStruct *f = get_field(env, field);
  jvalue val;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  val.s = value;

  set_instance_field(obj, f, val);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setInt(JNIEnv *env,
				    jobject field,
				    jobject obj,
				    jint value)
{
  FieldStruct *f = get_field(env, field);
  jvalue val;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  val.i = value;

  set_instance_field(obj, f, val);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setLong(JNIEnv *env,
				     jobject field,
				     jobject obj,
				     jlong value)
{
  FieldStruct *f = get_field(env, field);
  jvalue val;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  val.j = value;

  set_instance_field(obj, f, val);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setFloat(JNIEnv *env,
				      jobject field,
				      jobject obj,
				      jfloat value)
{
  FieldStruct *f = get_field(env, field);
  jvalue val;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  val.f = value;

  set_instance_field(obj, f, val);
}

JNIEXPORT void JNICALL
Java_java_lang_reflect_Field_setDouble(JNIEnv *env,
				       jobject field,
				       jobject obj,
				       jdouble value)
{
  FieldStruct *f = get_field(env, field);
  jvalue val;

  if (f->access_flags & ACC_FINAL)
    {
      jclass exception = (*env)->FindClass(env, "java/lang/IllegalAccessException");

      (*env)->ThrowNew(env, exception, "in java/lang/reflect/Field.setBoolean()");
      return;
    }

  val.d = value;

  set_instance_field(obj, f, val);
}
